import csv
import email
import logging
import os
import zipfile
from email import parser
from typing import BinaryIO
from typing import Tuple
from typing import Union

import click

log_file_path = ''


def get_log_record_extra_fields(record):
    """Taken from `common` repo logging module"""
    # The list contains all the attributes listed in
    # http://docs.python.org/library/logging.html#logrecord-attributes
    skip_list = (
        'args', 'asctime', 'created', 'exc_info', 'exc_text', 'filename',
        'funcName', 'id', 'levelname', 'levelno', 'lineno', 'module',
        'msecs', 'msecs', 'message', 'msg', 'name', 'pathname', 'process',
        'processName', 'relativeCreated', 'thread', 'threadName', 'extra',
        'stack_info', 'exc_type', 'exc_msg')

    easy_types = (str, bool, dict, float, int, list, type(None))

    fields = {}

    for key, value in record.__dict__.items():
        if key not in skip_list:
            if isinstance(value, easy_types):
                fields[key] = value
            else:
                fields[key] = repr(value)

    return fields


class ExtraFormatter(logging.Formatter):
    """Taken from `common` repo logging module"""

    def format(self, record):
        extra = get_log_record_extra_fields(record)

        if extra:
            extra_string = ', '.join([f'{field}: {value}' for field, value in sorted(extra.items())])
        else:
            extra_string = ''

        record.__dict__['extra'] = extra_string

        return super(ExtraFormatter, self).format(record)


def init_log(logger_name, debug_mode=False):
    global log_file_path
    cli_logger = logging.getLogger(logger_name)
    cli_logger.setLevel(logging.DEBUG)

    # file
    try:
        current_directory = os.getcwd()
        log_file_path = os.path.join(current_directory, 'intezer-analyze-cli.log')
        handler = logging.FileHandler(log_file_path)
        formatter = ExtraFormatter('%(asctime)s %(levelname)-8s %(module)s line: %(lineno)d: %(message)s. %(extra)s')

    except Exception:
        print('Failed to create logs directory, prints all logs to the screen')
        handler = logging.StreamHandler()
        formatter = ExtraFormatter('%(levelname)s %(message)s. %(extra)s', '%H:%M:%S')

    handler.setFormatter(formatter)
    handler.setLevel(logging.DEBUG)
    cli_logger.addHandler(handler)
    sdk_logger = logging.getLogger('intezer_sdk')
    sdk_logger.setLevel(logging.INFO)
    sdk_logger.addHandler(handler)

    # stderr
    if debug_mode and handler is not logging.StreamHandler:
        console_formatter = ExtraFormatter('%(levelname)s %(message)s. %(extra)s', '%H:%M:%S')
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(console_formatter)
        console_handler.setLevel(logging.DEBUG)
        cli_logger.addHandler(console_handler)
        sdk_logger.addHandler(console_handler)
        sdk_logger.setLevel(logging.DEBUG)


def is_supported_file(file_path):
    try:
        with open(file_path, 'rb') as f:
            byte = f.read(6)
            is_supported = (byte[:2] == b'MZ' or  # PE
                            byte[:4] == b'\x7fELF' or  # ELF
                            byte[:4] == b'dex\x0a' or  # Dex
                            byte[:4] == b'\x50\x4b\x03\x04' or  # Zip
                            byte[:3] == b'\x1f\x8b\x08' or  # Zip
                            byte[:6] == b'\x37\x7a\xbc\xaf\x27\x1c')  # 7-Zip
    except IOError:
        logging.info('No read permissions for file', extra=dict(file_path=file_path))
        return False

    if not is_supported and zipfile.is_zipfile(file_path):
        is_supported = is_apk(file_path)

    return is_supported


def is_eml_file(stream: BinaryIO) -> Tuple[bool, Union[str, None]]:
    mail_parser = email.parser.BytesParser()
    received_headers = ['To', 'Received']
    required_headers = ['From', 'Date']
    try:
        parsed_email = mail_parser.parse(stream, headersonly=True)
        date = parsed_email.get('Date')
        return all(header in parsed_email for header in required_headers) and any(
            header in parsed_email for header in received_headers
        ), date
    except Exception:
        pass
    finally:
        stream.seek(0)
    return False, None


def is_apk(file_path):
    try:
        with zipfile.ZipFile(file_path) as apk_zip:
            file_contents = apk_zip.namelist()
            return ('AndroidManifest.xml' in file_contents and
                    ('classes.dex' in file_contents or 'resources.arsc' in file_contents))
    except (OSError, zipfile.BadZipFile):
        logging.info('Error bad zip file')
        return False


def check_should_continue_for_large_dir(num_of_items, threshold):
    """
    Checks if there is an unusual amount of files in the directory, if so, asks the user if he's sure that he wants
    to continue. If the answer is no, aborting the operation
    :param num_of_items: The number of items in the dir
    :param threshold: The threshold amount required to do the check
    """
    if num_of_items <= threshold:
        return

    click.confirm('This directory contains more than {} files, are you sure you want to continue?'.format(threshold),
                  abort=True)


def indexes_list_to_csv(csv_path, indexes_list):
    file_path = os.path.join(csv_path, 'result.csv')
    with open(file_path, 'w') as csvFile:
        writer = csv.DictWriter(csvFile, fieldnames=['file_name', 'result'])
        writer.writeheader()
        writer.writerows(indexes_list)

    csvFile.close()


def export_to_csv(csv_file_path, items, keys=None):
    if not keys:
        keys = {key for code_item_data in items for key in code_item_data.keys()}

    sorted_keys = sorted(keys)

    with open(csv_file_path, 'w') as output_file:
        dict_writer = csv.DictWriter(output_file, sorted_keys)
        dict_writer.writeheader()
        dict_writer.writerows(items)


def is_hidden(path):
    if os.name == 'nt':
        try:
            import win32api
            import win32con
        except ImportError:
            return False
        attribute = win32api.GetFileAttributes(path)
        return attribute & (win32con.FILE_ATTRIBUTE_HIDDEN | win32con.FILE_ATTRIBUTE_SYSTEM)
    else:
        return os.path.basename(path).startswith('.')


if __name__ == '__main__':
    is_supported_file('/home/david/Downloads/lsass_pe.7z')



import logging
import os
from io import BytesIO
from typing import Optional
from email.utils import parsedate_to_datetime

import click
from intezer_sdk import api
from intezer_sdk import consts as sdk_consts
from intezer_sdk import errors as sdk_errors
from intezer_sdk.alerts import Alert
from intezer_sdk.analysis import FileAnalysis
from intezer_sdk.endpoint_analysis import EndpointAnalysis
from intezer_sdk.index import Index

from intezer_analyze_cli import key_store
from intezer_analyze_cli import utilities
from intezer_analyze_cli.config import default_config
from intezer_analyze_cli.utilities import is_hidden

logger = logging.getLogger('intezer_cli')


def login(api_key: str, api_url: str):
    try:
        if api_url:
            key_store.store_default_url(api_url)
        else:
            api_url = default_config.api_url
            key_store.delete_default_url()

        api.set_global_api(api_key, default_config.api_version, api_url)
        api.get_global_api().authenticate()
        key_store.store_api_key(api_key)
        click.echo('You have successfully logged in')
    except sdk_errors.InvalidApiKey:
        click.echo(f'Invalid API key error, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')
        raise click.Abort()


def analyze_file_command(file_path: str,
                         disable_dynamic_unpacking: bool,
                         disable_static_unpacking: bool,
                         code_item_type: str):
    if disable_dynamic_unpacking and not utilities.is_supported_file(file_path):
        click.echo('File is not PE, ELF, DEX or APK')
        return

    try:
        analysis = FileAnalysis(file_path=file_path,
                                code_item_type=code_item_type,
                                disable_dynamic_unpacking=disable_dynamic_unpacking,
                                disable_static_unpacking=disable_static_unpacking)
        analysis.send()
        analysis_page_url = default_config.file_analysis_url_template.format(
            system_url=default_config.api_url.replace('/api/', ''),
            analysis_id=analysis.analysis_id
        )
        click.echo(f'Analysis created. In order to check its result, go to: {analysis_page_url}')
    except sdk_errors.IntezerError as e:
        click.echo(f'Analyze error: {e}')


def analyze_directory_command(path: str,
                              disable_dynamic_unpacking: bool,
                              disable_static_unpacking: bool,
                              code_item_type: str,
                              ignore_directory_count_limit: bool):
    success_number = 0
    failed_number = 0
    unsupported_number = 0

    for root, dirs, files in os.walk(path):
        files = [f for f in files if not is_hidden(os.path.join(root, f))]
        dirs[:] = [d for d in dirs if not is_hidden(os.path.join(root, d))]

        number_of_files = len(files)
        if not ignore_directory_count_limit:
            utilities.check_should_continue_for_large_dir(number_of_files, default_config.unusual_amount_in_dir)
        if not files:
            continue

        with click.progressbar(length=number_of_files,
                               label='Sending files for analysis',
                               show_pos=True) as progressbar:
            for file_name in files:
                file_path = os.path.join(root, file_name)
                if disable_dynamic_unpacking and not utilities.is_supported_file(file_path):
                    unsupported_number += 1
                else:
                    try:
                        FileAnalysis(file_path=file_path,
                                     code_item_type=code_item_type,
                                     disable_dynamic_unpacking=disable_dynamic_unpacking,
                                     disable_static_unpacking=disable_static_unpacking).send()
                        success_number += 1
                    except sdk_errors.IntezerError as ex:
                        # We cannot continue analyzing the directory if the account is out of quota
                        if isinstance(ex, sdk_errors.InsufficientQuota):
                            logger.error('Failed to analyze %s', file_path)
                            raise

                        logger.exception('Error while analyzing directory')
                        failed_number += 1
                    except Exception:
                        logger.exception('Failed to analyze %s', file_path)
                        failed_number += 1

                progressbar.update(1)

    if success_number != 0:
        analyses_page_url = default_config.history_page_url_template.format(
            system_url=default_config.api_url.replace('/api/', ''),
            tab_name=default_config.file_analyses_tab_name
        )
        click.echo(f'{success_number} analysis created. In order to check their results, go to: {analyses_page_url}')

    if failed_number != 0:
        click.echo(f'{failed_number} analysis failed')

    if unsupported_number != 0:
        click.echo(f'{unsupported_number} unsupported files')


def analyze_by_txt_file_command(path: str):
    try:
        hashes = get_hashes_from_file(path)
        with click.progressbar(length=len(hashes),
                               label='Analyze files',
                               show_pos=True,
                               width=0) as progressbar:
            for file_hash in hashes:
                try:
                    FileAnalysis(file_hash=file_hash).send()
                except sdk_errors.HashDoesNotExistError:
                    click.echo(f'Hash: {file_hash} does not exist in the system')
                    logger.info('Hash not exists', extra=dict(file_hash=file_hash))
                except sdk_errors.IntezerError:
                    click.echo(f'Error occurred with hash: {file_hash}')
                    logger.exception('Error occurred with hash', extra=dict(file_hash=file_hash))
                progressbar.update(1)
            analyses_page_url = default_config.history_page_url_template.format(
                system_url=default_config.api_url.replace('/api/', ''),
                tab_name=default_config.file_analyses_tab_name
            )
            click.echo(f'analysis created. In order to check their results, go to: {analyses_page_url}')
    except IOError:
        click.echo(f'No read permissions for {path}')
        logger.exception('Error reading hashes file', extra=dict(path=path))
        raise click.Abort()


def index_by_txt_file_command(path: str, index_as: str, family_name: str):
    try:
        hashes = get_hashes_from_file(path)
        index_exceptions = []
        index_operations = []
        with click.progressbar(length=len(hashes),
                               label='Indexing files',
                               show_pos=True,
                               width=0) as upload_progress:
            for sha256 in hashes:
                index_operation, index_exception = index_hash_command(sha256, index_as, family_name)
                if index_operation:
                    index_operations.append((index_operation, sha256))
                else:
                    index_exceptions.append(index_exception)
                upload_progress.update(1)
        click.echo('Indexing sent')

        echo_exceptions(index_exceptions)
        index_exceptions = []
        with click.progressbar(length=len(index_operations),
                               label='Waiting for indexing to finish',
                               show_pos=True,
                               width=0) as index_progress:
            for index_operation, sha256 in index_operations:
                try:
                    index_operation.wait_for_completion()
                except sdk_errors.IntezerError as e:
                    index_exceptions.append(f'Failed to index hash: {sha256} error: {e}')
                    logger.exception('Failed to index hash', extra=dict(sha256=sha256))
                except sdk_errors.IndexFailed:
                    index_exceptions.append(f'Failed to index hash: {sha256} error: {e}')
                    logger.exception('Failed to index hash', extra=dict(sha256=sha256))
                index_progress.update(1)

        echo_exceptions(index_exceptions)

        private_index_page_url = default_config.history_page_url_template.format(
            system_url=default_config.api_url.replace('/api/', ''),
            tab_name=default_config.index_results_tab_name
        )
        click.echo(f'Index updated. In order to check their results, go to: {private_index_page_url}')

    except IOError:
        click.echo(f'No read permissions for {path}')
        logger.exception('Error reading hashes file', extra=dict(path=path))
        raise click.Abort()


def echo_exceptions(exceptions):
    if exceptions:
        click.echo('Some hashes failed')
        for exception in exceptions:
            click.echo(exception)


def get_hashes_from_file(path):
    with open(path, 'r') as file:
        hashes = [line.strip('\n') for line in file.readlines()]
        return hashes


def index_hash_command(sha256: str, index_as: str, family_name: Optional[str]):
    try:
        index_operation = Index(index_as=sdk_consts.IndexType.from_str(index_as),
                                sha256=sha256,
                                family_name=family_name)
        index_operation.send(wait=False)
        return index_operation, None
    except sdk_errors.IntezerError as e:
        logger.exception('Failed to index hash', extra=dict(sha256=sha256))
        return None, f'Index error: {e} Error occurred with hash: {sha256}'


def index_file_command(file_path: str, index_as: str, family_name: Optional[str]):
    if not utilities.is_supported_file(file_path):
        click.echo('File is not PE, ELF, DEX or APK')
        return
    try:
        index = Index(index_as=sdk_consts.IndexType.from_str(index_as), file_path=file_path, family_name=family_name)
        index.send(wait=True)
        click.echo(f'Finish index: {index.index_id} with status: {index.status}')
    except sdk_errors.IntezerError as e:
        logger.exception('Failed to index file', extra=dict(file_path=file_path))
        click.echo(f'Index error: {e}')


def index_directory_command(directory_path: str,
                            index_as: str,
                            family_name: Optional[str],
                            ignore_directory_count_limit: bool):
    indexes_results = []

    for root, dirs, files in os.walk(directory_path):
        files = [f for f in files if not is_hidden(os.path.join(root, f))]
        dirs[:] = [d for d in dirs if not is_hidden(os.path.join(root, d))]

        number_of_files = len(files)
        if not ignore_directory_count_limit:
            utilities.check_should_continue_for_large_dir(number_of_files, default_config.unusual_amount_in_dir)
        with click.progressbar(length=number_of_files,
                               label='Index files',
                               show_pos=True,
                               width=0) as progressbar:
            for file_name in files:
                file_path = os.path.join(root, file_name)

                if not utilities.is_supported_file(file_path):
                    click.echo(f'Could not open {file_name} because it is not a supported file type')
                    progressbar.update(1)
                    continue

                try:
                    index = Index(index_as=sdk_consts.IndexType.from_str(index_as),
                                  file_path=file_path,
                                  family_name=family_name)
                    index.send()
                    indexes_results.append({'file_name': file_name, 'index': index})
                except sdk_errors.IntezerError:
                    logger.exception('Failed to index file', extra=dict(file_name=file_name))
                    click.echo(f'Error occurred during indexing of {file_name}')
                    progressbar.update(1)

            for index_result in indexes_results:
                try:
                    index_result['index'].wait_for_completion()
                    click.echo(f'Index: {index_result["index"].index_id} ,'
                               f' File: {index_result["file_name"]} ,'
                               f' finished with status: {index_result["index"].status}')
                    progressbar.update(1)
                except Exception:
                    logger.exception('Failed to index file', extra=dict(file_name=index_result['file_name']))
                    click.echo(f'Error occurred during indexing of {index_result["file_name"]}')
                    progressbar.update(1)


def upload_offline_endpoint_scan(offline_scan_directory: str, force: bool = False, max_concurrent_uploads: int = 0):
    try:
        if not force and _was_directory_already_sent(offline_scan_directory):
            raise click.Abort()
        click.echo(f'Uploading: {os.path.basename(os.path.abspath(offline_scan_directory))}')
        endpoint_analysis = EndpointAnalysis(offline_scan_directory=offline_scan_directory,
                                             max_concurrent_uploads=max_concurrent_uploads)
        endpoint_analysis.send(wait=False)
        if not endpoint_analysis.analysis_id:
            raise RuntimeError('Error encountered while sending offline scan, server did not return analysis id')
        _create_analysis_id_file(offline_scan_directory, endpoint_analysis.analysis_id)

        endpoint_analysis_page_url = default_config.endpoint_analysis_url_template.format(
            system_url=default_config.api_url.replace('/api/', ''),
            endpoint_analysis_id=endpoint_analysis.analysis_id
        )

        click.echo(f'Analysis created. In order to check its result, go to: {endpoint_analysis_page_url}')

    except sdk_errors.IntezerError as e:
        click.echo(f'Analyze error: {e}')
        logger.exception('Failed to analyze offline scan')
        raise
    return endpoint_analysis.analysis_id


def upload_multiple_offline_endpoint_scans(offline_scans_root_directory: str,
                                           force: bool = False,
                                           max_concurrent_uploads: int = 0):
    success_number = 0
    failed_number = 0

    directories = _get_scan_subdirectories(offline_scans_root_directory)

    with click.progressbar(length=len(directories),
                           label='Sending offline endpoint scans for analysis',
                           show_pos=True) as progressbar:
        for scan_dir in directories:
            offline_scan_directory = os.path.join(offline_scans_root_directory, scan_dir)
            try:
                upload_offline_endpoint_scan(offline_scan_directory,
                                             force,
                                             max_concurrent_uploads=max_concurrent_uploads)
                success_number += 1
            except Exception as e:
                logger.exception(f'Error while analyzing directory {scan_dir}: {str(e)}')
                failed_number += 1
            finally:
                progressbar.update(1)

    if success_number != 0:
        endpoint_analyses_page_url = default_config.history_page_url_template.format(
            system_url=default_config.api_url.replace('/api/', ''),
            tab_name=default_config.endpoint_analyses_tab_name
        )
        click.echo(
            f'{success_number} analysis created. In order to check their results, go to: {endpoint_analyses_page_url}')
    if failed_number != 0:
        click.echo(f'{failed_number} offline endpoint scans failed to send')


def send_phishing_emails_from_directory_command(path: str,
                                                ignore_directory_count_limit: bool = False):
    success_number = 0
    failed_number = 0
    unsupported_number = 0
    emails_dates = []

    for root, dirs, files in os.walk(path):
        files = [f for f in files if not is_hidden(os.path.join(root, f))]

        number_of_files = len(files)
        if not ignore_directory_count_limit:
            utilities.check_should_continue_for_large_dir(number_of_files, default_config.unusual_amount_in_dir)
        if not files:
            continue

        with click.progressbar(length=number_of_files,
                               label='Sending files for analysis',
                               show_pos=True) as progressbar:
            for file_name in files:
                email_path = os.path.join(root, file_name)
                with open(email_path, 'rb') as email_file:
                    binary_data = BytesIO(email_file.read())
                is_eml, date = utilities.is_eml_file(binary_data)
                if not is_eml:
                    unsupported_number += 1
                    continue
                try:
                    Alert.send_phishing_email(raw_email=binary_data)
                    success_number += 1
                    if date:
                        try:
                            timestamp = parsedate_to_datetime(date).timestamp()
                            emails_dates.append(timestamp)
                        except Exception:
                            continue

                except sdk_errors.IntezerError as ex:
                    logger.exception('Error while analyzing directory')
                    failed_number += 1
                except Exception:
                    logger.exception(f'Failed to analyze {email_path}')
                    failed_number += 1

                progressbar.update(1)

    if success_number != 0:
        alerts_page_url = default_config.phishing_alerts_by_time_template.format(
            system_url=default_config.api_url.replace('/api/', '')
        )
        earliest_email_timestamp = int(min(emails_dates)) if emails_dates else None
        latest_email_timestamp = int(max(emails_dates)) if emails_dates else None
        if earliest_email_timestamp and latest_email_timestamp:
            alerts_page_url = f'{alerts_page_url}&start_time={earliest_email_timestamp}&end_time={latest_email_timestamp}'
        click.echo(f'{success_number} alerts created. In order to check their results, go to: {alerts_page_url}')

    if failed_number != 0:
        click.echo(f'{failed_number} scans failed')

    if unsupported_number != 0:
        click.echo(f'{unsupported_number} unsupported files')


def _get_scan_subdirectories(offline_scans_root_directory):
    directories = [d for d in os.listdir(offline_scans_root_directory) if
                   os.path.isdir(os.path.join(offline_scans_root_directory, d)) and
                   not is_hidden(os.path.join(offline_scans_root_directory, d))]
    for directory in ('files', 'fileless', 'memory_modules', 'logs'):
        if directory in directories:
            directories.remove(directory)
    return directories

def _was_directory_already_sent(path: str) -> bool:
    try:
        if os.path.isfile(os.path.join(path, 'analysis_id.txt')):
            with open(os.path.join(path, 'analysis_id.txt')) as f:
                analysis_id = f.read()

            endpoint_analysis_page_url = default_config.endpoint_analysis_url_template.format(
                system_url=default_config.api_url.replace('/api/', ''),
                endpoint_analysis_id=analysis_id
            )

            click.echo(f'Scan: {os.path.dirname(path)} has already been sent for analysis. '
                       f'See: {endpoint_analysis_page_url}')
            return True
    except Exception:
        logger.exception('Error while reading analysis_id.txt file in directory', extra=dict(path=path))
        click.echo(f'Error while reading analysis_id.txt file in directory {os.path.dirname(path)}')
        raise
    return False


def _create_analysis_id_file(directory: str, analysis_id: str):
    try:
        with open(os.path.join(directory, 'analysis_id.txt'), 'w') as f:
            f.write(analysis_id)
    except Exception:
        logger.exception('Could not create analysis_id.txt file', extra=dict(directory=directory))
        click.echo(f'Could not create analysis_id.txt file in {directory}')
        raise



import logging
import os

from intezer_analyze_cli.config import default_config as config_

logger = logging.getLogger('intezer_cli')


def get_key_file_path(key_file_name):
    if os.name == 'posix':
        return os.path.join(os.path.expanduser('~'), config_.key_dir_name, key_file_name)

    return os.path.join(os.path.expandvars('%APPDATA%'), config_.key_dir_name, key_file_name)


def delete_key(key_file_name):
    current_key = get_stored_key(key_file_name)
    if current_key:
        os.remove(get_key_file_path(key_file_name))


def store_key(key, key_file_name):
    current_key = get_stored_key(key_file_name)
    key_file_path = get_key_file_path(key_file_name)
    if current_key:
        os.remove(key_file_path)
        logger.info('Old key deleted')

    if not os.path.exists(os.path.dirname(key_file_path)):
        os.makedirs(os.path.dirname(key_file_path))

    with open(key_file_path, 'w') as f:
        f.write(key)
    logger.info('Key stored')


def get_stored_key(key_file_name):
    key_file_path = get_key_file_path(key_file_name)
    if not os.path.isfile(key_file_path):
        return None

    with open(get_key_file_path(key_file_name), 'r') as file:
        key = file.read()
        return key


def get_stored_api_key():
    return get_stored_key(config_.key_file_name)


def get_stored_default_url():
    return get_stored_key(config_.url_file_name)


def store_api_key(key):
    store_key(key, config_.key_file_name)


def store_default_url(key):
    store_key(key, config_.url_file_name)


def delete_default_url():
    delete_key(config_.url_file_name)



class Config(object):
    def __init__(self):
        # Client
        self.unusual_amount_in_dir = 1000
        self.verify_ssl = True

        # Urls
        self.api_url = 'https://analyze.intezer.com/api/'
        self.api_version = 'v2-0'
        self.file_analyses_tab_name = 'file'
        self.endpoint_analyses_tab_name = 'endpoint'
        self.index_results_tab_name = 'private_index'

        # URLs templates
        self.history_page_url_template = '{system_url}/history?tab={tab_name}'
        self.file_analysis_url_template = '{system_url}/analyses/{analysis_id}'
        self.endpoint_analysis_url_template = '{system_url}/endpoint-analyses/{endpoint_analysis_id}'
        self.phishing_alerts_by_time_template = '{system_url}/alerts/?sources=phishing_emails'

        # Key Store
        self.key_dir_name = '.intezer'
        self.key_file_name = 'key'
        self.url_file_name = 'url'

        # Other
        self.is_cloud = True


default_config = Config()



import logging
import os

import click
from intezer_sdk import api
from intezer_sdk import consts as sdk_consts
from intezer_sdk import errors as sdk_errors
from intezer_sdk.consts import CodeItemType

from intezer_analyze_cli import __version__
from intezer_analyze_cli import commands
from intezer_analyze_cli import key_store
from intezer_analyze_cli import utilities
from intezer_analyze_cli.config import default_config

utilities.init_log('intezer_cli', os.environ.get('INTEZER_DEBUG') == '1')
logger = logging.getLogger('intezer_cli')


def create_global_api():
    try:
        api_key = key_store.get_stored_api_key()
        api_url = key_store.get_stored_default_url()

        if not api_key:
            logger.exception('Cant find API key')
            click.echo('Cant find API key, please login')
            raise click.Abort()

        if api_url:
            default_config.api_url = api_url
            default_config.is_cloud = False

        api.set_global_api(api_key, default_config.api_version, default_config.api_url)
        sdk_consts.USER_AGENT += f'/CLI-{__version__}'

    except sdk_errors.InvalidApiKey:
        logger.exception('Invalid api key error')
        click.echo('Invalid API key error, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')
        raise click.Abort()


@click.group(context_settings=dict(help_option_names=['-h', '--help'], max_content_width=120),
             help=f'Intezer Labs Ltd. Intezer Analyze CLI {__version__}')
def main_cli():
    pass


@main_cli.command('login', short_help='Login to Intezer Analyze')
@click.argument('api_key', type=click.UUID)
@click.argument('api_url', required=False, default=None, type=click.STRING)
def login(api_key: str, api_url: str):
    """Login to Intezer Analyze to perform analyses.

    \b
    API_KEY: API key or invite code for Intezer Analyze.

    \b
    API_URL: Intezer Analyze URL in case you have on premise deployment.

    \b
    Example:
      $ intezer-analyze login edb45d954da54e8e980078001d8921cc
    """
    try:
        if api_url:
            if api_url[-1] != '/':
                api_url += '/'
            if not api_url.endswith('/api/'):
                api_url += 'api/'
        commands.login(str(api_key), api_url)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('analyze', short_help='Send a file or a directory for analysis')
@click.option('--no-unpacking', is_flag=True, help='Should the analysis skip unpacking')
@click.option('--no-static-extraction', is_flag=True, help='Should the analysis skip static extraction')
@click.option('--code-item-type', type=click.Choice([c.value for c in CodeItemType]), default=None,
              help='The type of the binary file uploaded')
@click.option('--ignore-directory-count-limit',
              is_flag=True,
              help='ignore directory count limit ({} files)'.format(default_config.unusual_amount_in_dir))
@click.argument('path', type=click.Path(exists=True))
def analyze(path: str,
            no_unpacking: bool,
            no_static_extraction: bool,
            code_item_type: str,
            ignore_directory_count_limit: bool):
    """ Send a file or a directory for analysis in Intezer Analyze.

    \b
    PATH: Path to file or directory to send the files inside for analysis.

    \b
    Examples:
      Send a single file for analysis:
      $ intezer-analyze analyze ~/files/threat.exe.sample
      \b
      Send all files in directory for analysis:
      $ intezer-analyze analyze ~/files/files-to-analyze
    """
    try:
        create_global_api()

        if not no_unpacking:
            no_unpacking = None
        if not no_static_extraction:
            no_static_extraction = None

        if os.path.isfile(path):
            commands.analyze_file_command(file_path=path,
                                          disable_dynamic_unpacking=no_unpacking,
                                          disable_static_unpacking=no_static_extraction,
                                          code_item_type=code_item_type)
        else:
            commands.analyze_directory_command(path=path,
                                               disable_dynamic_unpacking=no_unpacking,
                                               disable_static_unpacking=no_static_extraction,
                                               code_item_type=code_item_type,
                                               ignore_directory_count_limit=ignore_directory_count_limit)
    except click.Abort:
        raise
    except sdk_errors.InsufficientQuota:
        logger.exception('Insufficient quota')
        click.echo('Insufficient quota, please contact us at support@intezer.com ')
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('analyze_by_list', short_help='Send a text file with list of hashes')
@click.argument('path', type=click.Path(exists=True, dir_okay=False))
def analyze_by_list(path):
    """ Send a text file with hashes for analysis in Intezer Analyze.

    \b
    PATH: Path to txt file.

    \b
    Examples:
      Send txt file with hashes for analysis:
      $ intezer-analyze analyze_by_list ~/files/hashes.txt
    """
    try:
        create_global_api()

        commands.analyze_by_txt_file_command(path=path)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('index_by_list', short_help='Send a text file with list of hashes, verdict, family name if malicious')
@click.argument('path', type=click.Path(exists=True, dir_okay=False))
@click.option('--index-as', type=click.Choice(['malicious', 'trusted'], case_sensitive=True))
@click.argument('family_name', required=False, type=click.STRING, default=None)
def index_by_list(path: str, index_as: str, family_name: str):
    """
    Send a text file with hashes for indexing in Intezer Analyze.

    \b
    PATH: Path to a txt file with hashes

    \b
    Examples:
      $ intezer-analyze index_by_list ~/files/hashes.txt malicious family_name
      \b
    """
    try:
        index_type = sdk_consts.IndexType.from_str(index_as)

        if index_type == sdk_consts.IndexType.MALICIOUS and family_name is None:
            click.echo('family_name is mandatory if the index type is malicious')
            return

        create_global_api()

        commands.index_by_txt_file_command(path=path, index_as=index_as, family_name=family_name)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('index', short_help='index a file or a directory')
@click.argument('path', type=click.Path(exists=True))
@click.option('--index-as', type=click.Choice(['malicious', 'trusted'], case_sensitive=True))
@click.argument('family_name', required=False, type=click.STRING, default=None)
@click.option('--ignore-directory-count-limit',
              is_flag=True,
              help='ignore directory count limit ({} files)'.format(default_config.unusual_amount_in_dir))
def index(path: str, index_as: str, family_name: str, ignore_directory_count_limit: bool):
    """ Send a file or a directory for indexing

    \b
    PATH: Path to file or directory to index

    \b
    Examples:
      index a single file:
      $ intezer-analyze index ~/files/threat.exe.sample malicious family_name
      \b
      index all files in directory:
      $ intezer-analyze index ~/files/files-to-index trusted
    """
    try:
        index_type = sdk_consts.IndexType.from_str(index_as)

        if index_type == sdk_consts.IndexType.MALICIOUS and family_name is None:
            click.echo('family_name is mandatory if the index type is malicious')
            return

        create_global_api()

        if os.path.isfile(path):
            commands.index_file_command(file_path=path, index_as=index_as, family_name=family_name)
        else:
            commands.index_directory_command(directory_path=path,
                                             index_as=index_as,
                                             family_name=family_name,
                                             ignore_directory_count_limit=ignore_directory_count_limit)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('upload_endpoint_scan', short_help='upload a directory with offline endpoint scan results')
@click.argument('offline_scan_directory', type=click.Path(exists=True))
@click.option('--force', is_flag=True, default=False, help='Upload scan even if it was already uploaded')
@click.option('--max-concurrent', default=0, type=int, help='Maximum number of concurrent uploads.')
def upload_endpoint_scan(offline_scan_directory: str, force: bool, max_concurrent: int):
    """ Upload a directory with offline endpoint scan results


    OFFLINE_SCAN_DIRECTORY: Path to directory with offline endpoint scan results


    Examples:
      upload a directory with offline endpoint scan results:

      $ intezer-analyze upload_endpoint_scan /path/to/endpoint_scan_results
    """
    try:
        create_global_api()
        commands.upload_offline_endpoint_scan(offline_scan_directory=offline_scan_directory,
                                              force=force,
                                              max_concurrent_uploads=max_concurrent)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('upload_endpoint_scans_in_directory',
                  short_help='upload all subdirectories with offline endpoint scan results')
@click.argument('offline_scans_root_directory', type=click.Path(exists=True))
@click.option('--force', is_flag=True, default=False, help='Upload scans even if they were already uploaded')
@click.option('--max-concurrent', default=0, type=int, help='Maximum number of concurrent uploads.')
def upload_endpoint_scans_in_directory(offline_scans_root_directory: str, force: bool = False, max_concurrent: int = 0):
    """ Upload all subdirectories with offline endpoint scan results


    OFFLINE_SCANS_ROOT_DIRECTORY: Path to root directory containing offline endpoint scan results


    Examples:
      upload a directory with offline endpoint scan results:

      $ intezer-analyze upload_endpoint_scans_in_directory /path/to/endpoint_scan_results_root
    """
    try:
        create_global_api()
        commands.upload_multiple_offline_endpoint_scans(offline_scans_root_directory=offline_scans_root_directory,
                                                        force=force,
                                                        max_concurrent_uploads=max_concurrent)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


@main_cli.command('upload_emails_in_directory',
                  short_help='upload all subdirectories with .emal files')
@click.argument('emails_root_directory', type=click.Path(exists=True, file_okay=False, dir_okay=True))
@click.option('--ignore-directory-count-limit',
              is_flag=True,
              help='ignore directory count limit ({} files)'.format(default_config.unusual_amount_in_dir))
def upload_emails_in_directory(emails_root_directory: str, ignore_directory_count_limit: bool = False):
    """ Upload all subdirectories with .eml files to analyze


    UPLOAD_EMAILS_IN_DIRECTORY: Path to root directory containing the .eml fiels


    Examples:
      upload a directory with .eml files:

      $ intezer-analyze upload_emails_in_directory /path/to/emails_root_directory
    """
    try:
        create_global_api()
        commands.send_phishing_emails_from_directory_command(path=emails_root_directory,
                                                             ignore_directory_count_limit=ignore_directory_count_limit)
    except click.Abort:
        raise
    except Exception:
        logger.exception('Unexpected error occurred')
        click.echo('Unexpected error occurred, please contact us at support@intezer.com '
                   f'and attach the log file in {utilities.log_file_path}')


if __name__ == '__main__':
    try:
        main_cli()

    except Exception as e:
        logger.exception(f'Unexpected error occurred {e}')
        click.echo('Unexpected error occurred')



import os

from setuptools import setup


def rel(*xs):
    return os.path.join(os.path.abspath(os.path.dirname(__file__)), *xs)


with open(rel('intezer_analyze_cli', '__init__.py'), 'r') as f:
    version_marker = '__version__ = '
    for line in f:
        if line.startswith(version_marker):
            _, version = line.split(version_marker)
            version = version.strip().strip("'")
            break
    else:
        raise RuntimeError('Version marker not found.')

install_requires = [
    'click==7.1.2',
    'intezer-sdk>=1.15.2,<2'
]
tests_require = [
    'pytest==6.1.2',
    'responses==0.17.0'
]

with open('README.md') as f:
    long_description = f.read()

setup(
    name='intezer-analyze-cli',
    version=version,
    description='Client library for Intezer cloud service',
    author='Intezer Labs ltd.',
    classifiers=[
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12'
    ],
    keywords='intezer',
    packages=['intezer_analyze_cli'],
    install_requires=install_requires,
    tests_require=tests_require,
    entry_points='''
        [console_scripts]
        intezer-analyze=intezer_analyze_cli.cli:main_cli
    ''',
    license='Apache License v2',
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires='>=3.8',
    zip_safe=False
)
